/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * Entscheidungstheorie - Prof.Dr.Peters - Master of Science
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 6.0
 * Semestergroup: IG2
 */

/**
 * USABILITY:
 *  Copyright (c) 2006
 *  Thomas Woellert (Dipl.-Inf. (FH)) (thomas@woellert.net)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package net.woellert.viergewinnt.player;

import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import net.woellert.viergewinnt.constants.ApplicationConstants;
import net.woellert.viergewinnt.objects.Playfield;
import net.woellert.viergewinnt.objects.PlayfieldFrame;
import net.woellert.viergewinnt.objects.Playfield.SlotState;

/**
 * Class Description:
 * 		Human player implementation, who is able to select his/her next turn by mouse click
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 */
public class HumanPlayer extends AbstractPlayer implements MouseListener {

	// Boolean if the turn has been made (to avoid double turns by double-clicking)
	protected boolean turnMade = false;

	// The actual playfield
	protected Playfield field = null;

	// The players' state
	protected SlotState playerState = null;

	/**
	 * Constructor
	 */
	public HumanPlayer() {
		super();
	}

	@Override
	public void doTurn(Playfield field, SlotState playerState) {

		// Add the mouse listener so the human player can click
		PlayfieldFrame frame = field.getFrame();
		frame.addMouseListener(this);

		// Remember the field and player state
		turnMade = false;
		this.field = field;
		this.playerState = playerState;

		// Wait till the human made his/her turn
		while (!turnMade) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}

		// Remove the mouse listener
		frame.removeMouseListener(this);
	}

	/**
	 * MOUSE LISTENER INTERFACE METHODS
	 */

	/**
	 * Called when the user clicked
	 * @param e
	 */
	public void mouseClicked(MouseEvent e) {

		// Check if the turn has already been made
		if (turnMade) {
			return;
		}

		// Grab the coordiantes and determine the column
		Point p = e.getPoint();
		int col = (int) (p.getX() / (field.getFrame().getSize().getWidth() /  ApplicationConstants.PLAYFIELD_SIZE_COLS));

		// Check if there is still a free slot in the column
		if (field.getState(ApplicationConstants.PLAYFIELD_SIZE_ROWS - 1, col) == SlotState.EMPTY) {
			field.doTurn(col, playerState);
			turnMade = true;
		} else {
			// Invalid column
			turnMade = false;
		}
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
}
